@@ -5,7 +5,7 @@
 
     Manual pages builder.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -40,7 +40,7 @@ class ManualPageBuilder(Builder):
     """
     name = 'man'
     format = 'man'
-    epilog = 'The manual pages are in %(outdir)s.'
+    epilog = __('The manual pages are in %(outdir)s.')
 
     default_translator_class = ManualPageTranslator
     supported_image_types = []  # type: List[unicode]
@@ -48,8 +48,8 @@ class ManualPageBuilder(Builder):
     def init(self):
         # type: () -> None
         if not self.config.man_pages:
-            logger.warning('no "man_pages" config value found; no manual pages '
-                           'will be written')
+            logger.warning(__('no "man_pages" config value found; no manual pages '
+                              'will be written'))
 
     def get_outdated_docs(self):
         # type: () -> Union[unicode, List[unicode]]
@@ -69,7 +69,7 @@ class ManualPageBuilder(Builder):
             components=(docwriter,),
             read_config_files=True).get_default_values()
 
-        logger.info(bold('writing... '), nonl=True)
+        logger.info(bold(__('writing... ')), nonl=True)
 
         for info in self.config.man_pages:
             docname, name, description, authors, section = info
