@@ -5,7 +5,7 @@
 
     Toctree collector for sphinx.environment.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -15,6 +15,7 @@ from six import iteritems
 from sphinx import addnodes
 from sphinx.environment.adapters.toctree import TocTree
 from sphinx.environment.collectors import EnvironmentCollector
+from sphinx.locale import __
 from sphinx.transforms import SphinxContentsFilter
 from sphinx.util import url_re, logging
 
@@ -66,6 +67,7 @@ class TocTreeCollector(EnvironmentCollector):
         numentries = [0]  # nonlocal again...
 
         def traverse_in_section(node, cls):
+            # type: (nodes.Node, Any) -> List[nodes.Node]
             """Like traverse(), but stay within the same section."""
             result = []
             if isinstance(node, cls):
@@ -77,6 +79,7 @@ class TocTreeCollector(EnvironmentCollector):
             return result
 
         def build_toc(node, depth=1):
+            # type: (nodes.Node, int) -> List[nodes.Node]
             entries = []
             for sectionnode in node:
                 # find all toctree nodes in this section and add them
@@ -86,7 +89,7 @@ class TocTreeCollector(EnvironmentCollector):
                     onlynode = addnodes.only(expr=sectionnode['expr'])
                     blist = build_toc(sectionnode, depth)
                     if blist:
-                        onlynode += blist.children
+                        onlynode += blist.children  # type: ignore
                         entries.append(onlynode)
                     continue
                 if not isinstance(sectionnode, nodes.section):
@@ -145,6 +148,7 @@ class TocTreeCollector(EnvironmentCollector):
         env.toc_secnumbers = {}
 
         def _walk_toc(node, secnums, depth, titlenode=None):
+            # type: (nodes.Node, Dict, int, nodes.Node) -> None
             # titlenode is the title of the document, it will get assigned a
             # secnumber too, so that it shows up in next/prev/parent rellinks
             for subnode in node.children:
@@ -165,18 +169,20 @@ class TocTreeCollector(EnvironmentCollector):
                 elif isinstance(subnode, addnodes.compact_paragraph):
                     numstack[-1] += 1
                     if depth > 0:
-                        number = tuple(numstack)
+                        number = list(numstack)
+                        secnums[subnode[0]['anchorname']] = tuple(numstack)
                     else:
                         number = None
-                    secnums[subnode[0]['anchorname']] = number
-                    subnode[0]['secnumber'] = list(number)
+                        secnums[subnode[0]['anchorname']] = None
+                    subnode[0]['secnumber'] = number
                     if titlenode:
-                        titlenode['secnumber'] = list(number)
+                        titlenode['secnumber'] = number
                         titlenode = None
                 elif isinstance(subnode, addnodes.toctree):
                     _walk_toctree(subnode, depth)
 
         def _walk_toctree(toctreenode, depth):
+            # type: (nodes.Node, int) -> None
             if depth == 0:
                 return
             for (title, ref) in toctreenode['entries']:
@@ -184,8 +190,8 @@ class TocTreeCollector(EnvironmentCollector):
                     # don't mess with those
                     continue
                 elif ref in assigned:
-                    logger.warning('%s is already assigned section numbers '
-                                   '(nested numbered toctree?)', ref,
+                    logger.warning(__('%s is already assigned section numbers '
+                                      '(nested numbered toctree?)'), ref,
                                    location=toctreenode, type='toc', subtype='secnum')
                 elif ref in env.tocs:
                     secnums = env.toc_secnumbers[ref] = {}
@@ -216,9 +222,19 @@ class TocTreeCollector(EnvironmentCollector):
         assigned = set()  # type: Set[unicode]
         old_fignumbers = env.toc_fignumbers
         env.toc_fignumbers = {}
-        fignum_counter = {}  # type: Dict[unicode, Dict[Tuple[int], int]]
+        fignum_counter = {}  # type: Dict[unicode, Dict[Tuple[int, ...], int]]
+
+        def get_figtype(node):
+            # type: (nodes.Node) -> unicode
+            for domain in env.domains.values():
+                figtype = domain.get_enumerable_node_type(node)
+                if figtype:
+                    return figtype
+
+            return None
 
         def get_section_number(docname, section):
+            # type: (unicode, nodes.Node) -> Tuple[int, ...]
             anchorname = '#' + section['ids'][0]
             secnumbers = env.toc_secnumbers.get(docname, {})
             if anchorname in secnumbers:
@@ -229,6 +245,7 @@ class TocTreeCollector(EnvironmentCollector):
             return secnum or tuple()
 
         def get_next_fignumber(figtype, secnum):
+            # type: (unicode, Tuple[int, ...]) -> Tuple[int, ...]
             counter = fignum_counter.setdefault(figtype, {})
 
             secnum = secnum[:env.config.numfig_secnum_depth]
@@ -236,6 +253,7 @@ class TocTreeCollector(EnvironmentCollector):
             return secnum + (counter[secnum],)
 
         def register_fignumber(docname, secnum, figtype, fignode):
+            # type: (unicode, Tuple[int], unicode, nodes.Node) -> None
             env.toc_fignumbers.setdefault(docname, {})
             fignumbers = env.toc_fignumbers[docname].setdefault(figtype, {})
             figure_id = fignode['ids'][0]
@@ -243,6 +261,7 @@ class TocTreeCollector(EnvironmentCollector):
             fignumbers[figure_id] = get_next_fignumber(figtype, secnum)
 
         def _walk_doctree(docname, doctree, secnum):
+            # type: (unicode, nodes.Node, Tuple[int, ...]) -> None
             for subnode in doctree.children:
                 if isinstance(subnode, nodes.section):
                     next_secnum = get_section_number(docname, subnode)
@@ -261,20 +280,21 @@ class TocTreeCollector(EnvironmentCollector):
 
                     continue
 
-                figtype = env.get_domain('std').get_figtype(subnode)
+                figtype = get_figtype(subnode)
                 if figtype and subnode['ids']:
                     register_fignumber(docname, secnum, figtype, subnode)
 
                 _walk_doctree(docname, subnode, secnum)
 
         def _walk_doc(docname, secnum):
+            # type: (unicode, Tuple[int]) -> None
             if docname not in assigned:
                 assigned.add(docname)
                 doctree = env.get_doctree(docname)
                 _walk_doctree(docname, doctree, secnum)
 
         if env.config.numfig:
-            _walk_doc(env.config.master_doc, tuple())
+            _walk_doc(env.config.master_doc, tuple())  # type: ignore
             for docname, fignums in iteritems(env.toc_fignumbers):
                 if fignums != old_fignumbers.get(docname):
                     rewrite_needed.append(docname)
