@@ -5,14 +5,22 @@
 
     Allow reference sections by :ref: role using its title.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from docutils import nodes
+
+from sphinx.locale import __
 from sphinx.util import logging
 from sphinx.util.nodes import clean_astext
 
+if False:
+    # For type annotation
+    from typing import Any, Dict  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+
+
 logger = logging.getLogger(__name__)
 
 if False:
@@ -28,15 +36,16 @@ def register_sections_as_label(app, document):
     for node in document.traverse(nodes.section):
         labelid = node['ids'][0]
         docname = app.env.docname
+        ref_name = getattr(node[0], 'rawsource', node[0].astext())
         if app.config.autosectionlabel_prefix_document:
-            name = nodes.fully_normalize_name(docname + ':' + node[0].astext())
+            name = nodes.fully_normalize_name(docname + ':' + ref_name)
         else:
-            name = nodes.fully_normalize_name(node[0].astext())
+            name = nodes.fully_normalize_name(ref_name)
         sectname = clean_astext(node[0])
 
         if name in labels:
-            logger.warning('duplicate label %s, ' % name + 'other instance '
-                           'in ' + app.env.doc2path(labels[name][0]),
+            logger.warning(__('duplicate label %s, other instance in %s'),
+                           name, app.env.doc2path(labels[name][0]),
                            location=node)
 
         anonlabels[name] = docname, labelid
