@@ -6,7 +6,7 @@
     Mimic doctest by automatically executing code snippets and checking
     their results.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 from __future__ import absolute_import
@@ -19,18 +19,19 @@ import time
 from os import path
 
 from docutils import nodes
-from docutils.parsers.rst import Directive, directives
+from docutils.parsers.rst import directives
 from packaging.specifiers import SpecifierSet, InvalidSpecifier
 from packaging.version import Version
 from six import itervalues, StringIO, binary_type, text_type, PY2
 
 import sphinx
 from sphinx.builders import Builder
-from sphinx.locale import _
+from sphinx.locale import __
 from sphinx.util import force_decode, logging
 from sphinx.util.console import bold  # type: ignore
+from sphinx.util.docutils import SphinxDirective
 from sphinx.util.nodes import set_source_info
-from sphinx.util.osutil import fs_encoding
+from sphinx.util.osutil import fs_encoding, relpath
 
 if False:
     # For type annotation
@@ -77,7 +78,7 @@ def is_allowed_version(spec, version):
 
 # set up the necessary directives
 
-class TestDirective(Directive):
+class TestDirective(SphinxDirective):
     """
     Base class for doctest-related directives.
     """
@@ -129,12 +130,12 @@ class TestDirective(Directive):
                 prefix, option_name = option[0], option[1:]
                 if prefix not in '+-':
                     self.state.document.reporter.warning(
-                        _("missing '+' or '-' in '%s' option.") % option,
+                        __("missing '+' or '-' in '%s' option.") % option,
                         line=self.lineno)
                     continue
                 if option_name not in doctest.OPTIONFLAGS_BY_NAME:
                     self.state.document.reporter.warning(
-                        _("'%s' is not a valid option.") % option_name,
+                        __("'%s' is not a valid option.") % option_name,
                         line=self.lineno)
                     continue
                 flag = doctest.OPTIONFLAGS_BY_NAME[option[1:]]
@@ -142,23 +143,25 @@ class TestDirective(Directive):
         if self.name == 'doctest' and 'pyversion' in self.options:
             try:
                 spec = self.options['pyversion']
-                python_version = '.'.join(str(v) for v in sys.version_info[:3])
+                python_version = '.'.join([str(v) for v in sys.version_info[:3]])
                 if not is_allowed_version(spec, python_version):
                     flag = doctest.OPTIONFLAGS_BY_NAME['SKIP']
                     node['options'][flag] = True  # Skip the test
             except InvalidSpecifier:
                 self.state.document.reporter.warning(
-                    _("'%s' is not a valid pyversion option") % spec,
+                    __("'%s' is not a valid pyversion option") % spec,
                     line=self.lineno)
+        if 'skipif' in self.options:
+            node['skipif'] = self.options['skipif']
         return [node]
 
 
 class TestsetupDirective(TestDirective):
-    option_spec = {}  # type: Dict
+    option_spec = {'skipif': directives.unchanged_required}  # type: Dict
 
 
 class TestcleanupDirective(TestDirective):
-    option_spec = {}  # type: Dict
+    option_spec = {'skipif': directives.unchanged_required}  # type: Dict
 
 
 class DoctestDirective(TestDirective):
@@ -166,6 +169,7 @@ class DoctestDirective(TestDirective):
         'hide': directives.flag,
         'options': directives.unchanged,
         'pyversion': directives.unchanged_required,
+        'skipif': directives.unchanged_required,
     }
 
 
@@ -173,6 +177,7 @@ class TestcodeDirective(TestDirective):
     option_spec = {
         'hide': directives.flag,
         'pyversion': directives.unchanged_required,
+        'skipif': directives.unchanged_required,
     }
 
 
@@ -181,6 +186,7 @@ class TestoutputDirective(TestDirective):
         'hide': directives.flag,
         'options': directives.unchanged,
         'pyversion': directives.unchanged_required,
+        'skipif': directives.unchanged_required,
     }
 
 
@@ -214,7 +220,7 @@ class TestGroup(object):
             if self.tests and len(self.tests[-1]) == 2:
                 self.tests[-1][1] = code
         else:
-            raise RuntimeError('invalid TestCode type')
+            raise RuntimeError(__('invalid TestCode type'))
 
     def __repr__(self):  # type: ignore
         # type: () -> unicode
@@ -275,8 +281,8 @@ class DocTestBuilder(Builder):
     Runs test snippets in the documentation.
     """
     name = 'doctest'
-    epilog = ('Testing of doctests in the sources finished, look at the '
-              'results in %(outdir)s/output.txt.')
+    epilog = __('Testing of doctests in the sources finished, look at the '
+                'results in %(outdir)s/output.txt.')
 
     def init(self):
         # type: () -> None
@@ -372,7 +378,7 @@ Doctest summary
         """Try to get the file which actually contains the doctest, not the
         filename of the document it's included in."""
         try:
-            filename = path.relpath(node.source, self.env.srcdir)\
+            filename = relpath(node.source, self.env.srcdir)\
                 .rsplit(':docstring of ', maxsplit=1)[0]
         except Exception:
             filename = self.env.doc2path(docname, base=None)
@@ -397,6 +403,20 @@ Doctest summary
             return node.line - 1
         return None
 
+    def skipped(self, node):
+        # type: (nodes.Element) -> bool
+        if 'skipif' not in node:
+            return False
+        else:
+            condition = node['skipif']
+            context = {}  # type: Dict[str, Any]
+            if self.config.doctest_global_setup:
+                exec(self.config.doctest_global_setup, context)
+            should_skip = eval(condition, context)
+            if self.config.doctest_global_cleanup:
+                exec(self.config.doctest_global_cleanup, context)
+            return should_skip
+
     def test_doc(self, docname, doctree):
         # type: (unicode, nodes.Node) -> None
         groups = {}  # type: Dict[unicode, TestGroup]
@@ -423,11 +443,14 @@ Doctest summary
                 return isinstance(node, (nodes.literal_block, nodes.comment)) \
                     and 'testnodetype' in node
         for node in doctree.traverse(condition):
+            if self.skipped(node):
+                continue
+
             source = node['test'] if 'test' in node else node.astext()
             filename = self.get_filename_for_node(node, docname)
             line_number = self.get_line_number(node)
             if not source:
-                logger.warning('no code/output in %s block at %s:%s',
+                logger.warning(__('no code/output in %s block at %s:%s'),
                                node.get('testnodetype', 'doctest'),
                                filename, line_number)
             code = TestCode(source, type=node.get('testnodetype', 'doctest'),
@@ -518,7 +541,7 @@ Doctest summary
                         doctest_encode(code[0].code, self.env.config.source_encoding), {},  # type: ignore  # NOQA
                         group.name, code[0].filename, code[0].lineno)
                 except Exception:
-                    logger.warning('ignoring invalid doctest code: %r', code[0].code,
+                    logger.warning(__('ignoring invalid doctest code: %r'), code[0].code,
                                    location=(code[0].filename, code[0].lineno))
                     continue
                 if not test.examples:
