@@ -5,7 +5,7 @@
 
     Theming support for HTML builders.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -133,7 +133,7 @@ class Theme(object):
 
         for option, value in iteritems(overrides):
             if option not in options:
-                logger.warning('unsupported theme option %r given' % option)
+                logger.warning(__('unsupported theme option %r given') % option)
             else:
                 options[option] = value
 
