@@ -5,7 +5,7 @@
 
     Utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 from __future__ import absolute_import
@@ -18,18 +18,20 @@ import sys
 import tempfile
 import traceback
 import unicodedata
+import warnings
 from codecs import BOM_UTF8
 from collections import deque
 from datetime import datetime
+from hashlib import md5
 from os import path
 from time import mktime, strptime
-from typing import TYPE_CHECKING
 
 from docutils.utils import relative_path
 from six import text_type, binary_type, itervalues
 from six.moves import range
 from six.moves.urllib.parse import urlsplit, urlunsplit, quote_plus, parse_qsl, urlencode
 
+from sphinx.deprecation import RemovedInSphinx30Warning
 from sphinx.errors import PycodeError, SphinxParallelError, ExtensionError
 from sphinx.util import logging
 from sphinx.util.console import strip_colors, colorize, bold, term_width_line  # type: ignore
@@ -47,7 +49,8 @@ from sphinx.util.nodes import (   # noqa
     caption_ref_re)
 from sphinx.util.matching import patfilter  # noqa
 
-if TYPE_CHECKING:
+if False:
+    # For type annotation
     from typing import Any, Callable, Dict, IO, Iterable, Iterator, List, Pattern, Sequence, Set, Tuple, Union  # NOQA
 
 
@@ -165,6 +168,37 @@ class FilenameUniqDict(dict):
         self._existing = state
 
 
+class DownloadFiles(dict):
+    """A special dictionary for download files.
+
+    .. important:: This class would be refactored in nearly future.
+                   Hence don't hack this directly.
+    """
+
+    def add_file(self, docname, filename):
+        # type: (unicode, unicode) -> None
+        if filename not in self:
+            digest = md5(filename.encode('utf-8')).hexdigest()
+            dest = '%s/%s' % (digest, os.path.basename(filename))
+            self[filename] = (set(), dest)
+
+        self[filename][0].add(docname)
+        return self[filename][1]
+
+    def purge_doc(self, docname):
+        # type: (unicode) -> None
+        for filename, (docs, dest) in list(self.items()):
+            docs.discard(docname)
+            if not docs:
+                del self[filename]
+
+    def merge_other(self, docnames, other):
+        # type: (Set[unicode], Dict[unicode, Tuple[Set[unicode], Any]]) -> None
+        for filename, (docs, dest) in other.items():
+            for docname in docs & set(docnames):
+                self.add_file(docname, filename)
+
+
 def copy_static_entry(source, targetdir, builder, context={},
                       exclude_matchers=(), level=0):
     # type: (unicode, unicode, Any, Dict, Tuple[Callable, ...], int) -> None
@@ -172,6 +206,9 @@ def copy_static_entry(source, targetdir, builder, context={},
 
     Handles all possible cases of files, directories and subdirectories.
     """
+    warnings.warn('sphinx.util.copy_static_entry is deprecated for removal',
+                  RemovedInSphinx30Warning, stacklevel=2)
+
     if exclude_matchers:
         relpath = relative_path(path.join(builder.srcdir, 'dummy'), source)
         for matcher in exclude_matchers:
@@ -264,7 +301,9 @@ def get_module_source(modname):
             raise PycodeError('error getting filename for %r' % filename, err)
     if filename is None and loader:
         try:
-            return 'string', loader.get_source(modname)
+            filename = loader.get_source(modname)
+            if filename:
+                return 'string', filename
         except Exception as err:
             raise PycodeError('error getting source for %r' % modname, err)
     if filename is None:
@@ -277,6 +316,12 @@ def get_module_source(modname):
             filename += 'w'
     elif not (lfilename.endswith('.py') or lfilename.endswith('.pyw')):
         raise PycodeError('source is not a .py file: %r' % filename)
+    elif ('.egg' + os.path.sep) in filename:
+        pat = '(?<=\\.egg)' + re.escape(os.path.sep)
+        eggpath, _ = re.split(pat, filename, 1)
+        if path.isfile(eggpath):
+            return 'file', filename
+
     if not path.isfile(filename):
         raise PycodeError('source file is not present: %r' % filename)
     return 'file', filename
@@ -335,7 +380,7 @@ def detect_encoding(readline):
         except UnicodeDecodeError:
             return None
 
-        matches = _coding_re.findall(line_string)  # type: ignore
+        matches = _coding_re.findall(line_string)
         if not matches:
             return None
         return get_normal_name(matches[0])
@@ -553,12 +598,11 @@ def import_object(objname, source=None):
 
 def encode_uri(uri):
     # type: (unicode) -> unicode
-    split = list(urlsplit(uri))  # type: Any
+    split = list(urlsplit(uri))  # type: List[unicode]
     split[1] = split[1].encode('idna').decode('ascii')
-    split[2] = quote_plus(split[2].encode('utf-8'), '/').decode('ascii')
-    query = list((q, quote_plus(v.encode('utf-8')))
-                 for (q, v) in parse_qsl(split[3]))
-    split[3] = urlencode(query).decode('ascii')
+    split[2] = quote_plus(split[2].encode('utf-8'), '/')
+    query = list((q, v.encode('utf-8')) for (q, v) in parse_qsl(split[3]))
+    split[3] = urlencode(query)
     return urlunsplit(split)
 
 
@@ -609,6 +653,7 @@ def status_iterator(iterable, summary, color="darkgreen", length=0, verbosity=0,
 
 
 def epoch_to_rfc1123(epoch):
+    # type: (float) -> unicode
     """Convert datetime format epoch to RFC1123."""
     from babel.dates import format_datetime
 
@@ -618,10 +663,12 @@ def epoch_to_rfc1123(epoch):
 
 
 def rfc1123_to_epoch(rfc1123):
+    # type: (str) -> float
     return mktime(strptime(rfc1123, '%a, %d %b %Y %H:%M:%S %Z'))
 
 
 def xmlname_checker():
+    # type: () -> Pattern
     # https://www.w3.org/TR/REC-xml/#NT-Name
     # Only Python 3.3 or newer support character code in regular expression
     name_start_chars = [
@@ -640,6 +687,7 @@ def xmlname_checker():
     ]
 
     def convert(entries, splitter=u'|'):
+        # type: (Any, unicode) -> unicode
         results = []
         for entry in entries:
             if isinstance(entry, list):
