@@ -5,7 +5,7 @@
 
     Helpers for inspecting Python modules.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 from __future__ import absolute_import
@@ -21,12 +21,15 @@ from six import PY2, PY3, StringIO, binary_type, string_types, itervalues
 from six.moves import builtins
 
 from sphinx.util import force_decode
+from sphinx.util import logging
 from sphinx.util.pycompat import NoneType
 
 if False:
     # For type annotation
     from typing import Any, Callable, Dict, List, Tuple, Type  # NOQA
 
+logger = logging.getLogger(__name__)
+
 memory_address_re = re.compile(r' at 0x[0-9a-f]{8,16}(?=>)', re.IGNORECASE)
 
 
@@ -262,8 +265,19 @@ def object_description(object):
         except Exception:
             pass  # Cannot sort dict keys, fall back to generic repr
         else:
-            items = ("%r: %r" % (key, object[key]) for key in sorted_keys)
+            items = ("%s: %s" %
+                     (object_description(key), object_description(object[key]))
+                     for key in sorted_keys)
             return "{%s}" % ", ".join(items)
+    if isinstance(object, set):
+        try:
+            sorted_values = sorted(object)
+        except TypeError:
+            pass  # Cannot sort set values, fall back to generic repr
+        else:
+            template = "{%s}" if PY3 else "set([%s])"
+            return template % ", ".join(object_description(x)
+                                        for x in sorted_values)
     try:
         s = repr(object)
     except Exception:
@@ -346,6 +360,9 @@ class Signature(object):
         try:
             self.annotations = typing.get_type_hints(subject)  # type: ignore
         except Exception:
+            # get_type_hints() does not support some kind of objects like partial,
+            # ForwardRef and so on.  For them, it raises an exception. In that case,
+            # we try to build annotations from argspec.
             self.annotations = {}
 
         if bound_method:
@@ -626,6 +643,7 @@ else:
     # of Python 3.5
 
     def _findclass(func):
+        # type: (Any) -> Any
         cls = sys.modules.get(func.__module__)
         if cls is None:
             return None
@@ -639,6 +657,7 @@ else:
         return cls
 
     def _finddoc(obj):
+        # type: (Any) -> unicode
         if inspect.isclass(obj):
             for base in obj.__mro__:
                 if base is not object:
@@ -699,6 +718,7 @@ else:
         return None
 
     def _getdoc(object):
+        # type: (Any) -> unicode
         """Get the documentation string for an object.
 
         All tabs are expanded to spaces.  To clean up docstrings that are
