@@ -5,11 +5,11 @@
 
     Inventory utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-import re
 import os
+import re
 import zlib
 
 from six import PY3
@@ -143,11 +143,11 @@ class InventoryFile(object):
             if not m:
                 continue
             name, type, prio, location, dispname = m.groups()
-            if type == 'py:module' and type in invdata and \
-                    name in invdata[type]:  # due to a bug in 1.1 and below,
-                                            # two inventory entries are created
-                                            # for Python modules, and the first
-                                            # one is correct
+            if type == 'py:module' and type in invdata and name in invdata[type]:
+                # due to a bug in 1.1 and below,
+                # two inventory entries are created
+                # for Python modules, and the first
+                # one is correct
                 continue
             if location.endswith(u'$'):
                 location = location[:-1] + name
